@@ -8,7 +8,7 @@
  * 
  * orcania.h: public header file
  * 
- * Copyright 2015-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2022 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -38,6 +38,11 @@ extern "C"
 #include <stdio.h>
 #include <stdlib.h>
 
+struct _o_datum {
+  size_t          size;
+  unsigned char * data;
+};
+
 /**
  * @defgroup str string functions
  * These functions are used for string manipulation
@@ -202,6 +207,12 @@ char * o_strrchr(const char * haystack, int c);
  */
 const char * o_strrnchr(const char * haystack, size_t len, char c);
 
+/**
+ * o_strnullempty
+ * return true if s is NULL or empty string, false otherwise
+ */
+int o_strnullempty(const char * s);
+
 /**
  * o_strlen
  * a modified version of strlen that don't crash when s is NULL
@@ -216,12 +227,13 @@ size_t o_strlen(const char * s);
  * @param str the string to trim
  * @return the trimmed str
  */
-char * trimwhitespace(char * str);
+const char * trimwhitespace(char * str);
 
 /**
  * Remove string of beginning and ending given character
  * the string str will be modified
  * @param str the string to trim
+ * @param to_remove the character to trim
  * @return the trimmed str
  */
 char * trimcharacter(char * str, char to_remove);
@@ -310,6 +322,7 @@ int string_array_has_value_case(const char ** array, const char * needle);
  * Check if an array of string has a specified value, case sensitive, limit to len characters
  * @param array an array of char * with NULL in the last element
  * @param needle the value to look for in array
+ * @param len the length of needle
  * @return 1 if needle exists in array, 0 otherwise
  */
 int string_array_has_value_n(const char ** array, const char * needle, size_t len);
@@ -318,6 +331,7 @@ int string_array_has_value_n(const char ** array, const char * needle, size_t le
  * Check if an array of string has a specified value, case insensitive, limit to len characters
  * @param array an array of char * with NULL in the last element
  * @param needle the value to look for in array
+ * @param len the length of needle
  * @return 1 if needle exists in array, 0 otherwise
  */
 int string_array_has_value_ncase(const char ** array, const char * needle, size_t len);
@@ -516,11 +530,11 @@ void o_get_alloc_funcs(o_malloc_t * malloc_fn, o_realloc_t * realloc_fn, o_free_
 
 /**
  * o_base64_encode - Base64 encode
- * @param src: Data to be encoded
- * @param len: Length of the data to be encoded
- * @param out: Pointer to output variable
- * @param out_len: Pointer to output length variable
- * @return : 1 on success, 0 on failure
+ * @param src Data to be encoded
+ * @param len Length of the data to be encoded
+ * @param out Pointer to output variable, you might add up to 2 bytes to out length than expected for safety
+ * @param out_len Pointer to output length variable
+ * @return 1 on success, 0 on failure
  *
  * The nul terminator is not included in out_len.
  */
@@ -528,11 +542,11 @@ int o_base64_encode(const unsigned char * src, size_t len, unsigned char * out,
 
 /**
  * o_base64_decode - Base64 decode
- * @param src: Data to be decoded
- * @param len: Length of the data to be decoded
- * @param out: Pointer to output variable
- * @param out_len: Pointer to output length variable
- * @return : 1 on success, 0 on failure
+ * @param src Data to be decoded
+ * @param len Length of the data to be decoded
+ * @param out Pointer to output variable, you might add up to 2 bytes to out length than expected for safety
+ * @param out_len Pointer to output length variable
+ * @return 1 on success, 0 on failure
  *
  * The nul terminator is not included in out_len.
  */
@@ -540,11 +554,11 @@ int o_base64_decode(const unsigned char *src, size_t len, unsigned char * out, s
 
 /**
  * o_base64url_encode - Base64url encode (url format)
- * @param src: Data to be encoded
- * @param len: Length of the data to be encoded
- * @param out: Pointer to output variable
- * @param out_len: Pointer to output length variable
- * @return : 1 on success, 0 on failure
+ * @param src Data to be encoded
+ * @param len Length of the data to be encoded
+ * @param out Pointer to output variable, you might add up to 2 bytes to out length than expected for safety
+ * @param out_len Pointer to output length variable
+ * @return 1 on success, 0 on failure
  *
  * The nul terminator is not included in out_len.
  */
@@ -552,23 +566,23 @@ int o_base64url_encode(const unsigned char * src, size_t len, unsigned char * ou
 
 /**
  * o_base64url_decode - Base64 decode (url format)
- * @param src: Data to be decoded
- * @param len: Length of the data to be decoded
- * @param out: Pointer to output variable
- * @param out_len: Pointer to output length variable
- * @return : 1 on success, 0 on failure
+ * @param src Data to be decoded
+ * @param len Length of the data to be decoded
+ * @param out Pointer to output variable, you might add up to 2 bytes to out length than expected for safety
+ * @param out_len Pointer to output length variable
+ * @return 1 on success, 0 on failure
  *
  * The nul terminator is not included in out_len.
  */
 int o_base64url_decode(const unsigned char *src, size_t len, unsigned char * out, size_t * out_len);
 
 /**
- * o_base64url_2_base64 - Convert a base64 url format to base64 format
- * @param src: Data to be decoded
- * @param len: Length of the data to be decoded
- * @param out: Pointer to output variable
- * @param out_len: Pointer to output length variable
- * @return : 1 on success, 0 on failure
+ * o_base64url_2_base64 - Convert a base64 url format to base64 format - Does not check if src is a valid base64
+ * @param src Data to be decoded
+ * @param len Length of the data to be decoded
+ * @param out Pointer to output variable
+ * @param out_len Pointer to output length variable
+ * @return 1 on success, 0 on failure
  *
  * The nul terminator is not included in out_len.
  * out must be at least len+2
@@ -576,18 +590,90 @@ int o_base64url_decode(const unsigned char *src, size_t len, unsigned char * out
 int o_base64url_2_base64(const unsigned char *src, size_t len, unsigned char * out, size_t * out_len);
 
 /**
- * o_base64url_2_base64 - Convert a base64 format to base64 url format
- * @param src: Data to be decoded
- * @param len: Length of the data to be decoded
- * @param out: Pointer to output variable
- * @param out_len: Pointer to output length variable
- * @return : 1 on success, 0 on failure
+ * o_base64_2_base64url - Convert a base64 url format to base64 format - Does not check if src is a valid base64url
+ * @param src Data to be decoded
+ * @param len Length of the data to be decoded
+ * @param out Pointer to output variable
+ * @param out_len Pointer to output length variable
+ * @return 1 on success, 0 on failure
  *
  * The nul terminator is not included in out_len.
  * out must be at least len+2
  */
 int o_base64_2_base64url(const unsigned char *src, size_t len, unsigned char * out, size_t * out_len);
 
+/**
+ * o_base64_encode_alloc - Base64 encode - allocate data result in a struct _o_datum * 
+ * @param src Data to be encoded
+ * @param len Length of the data to be encoded
+ * @param dat Pointer to a struct _o_datum * to store result
+ * @return 1 on success, 0 on failure
+ *
+ * dat->data must be o_free'd after use
+ * The nul terminator is not included in dat->data.
+ */
+int o_base64_encode_alloc(const unsigned char * src, size_t len, struct _o_datum * dat);
+
+/**
+ * o_base64_decode_alloc - Base64 decode - allocate data result in a struct _o_datum * 
+ * @param src Data to be decoded
+ * @param len Length of the data to be decoded
+ * @param dat Pointer to a struct _o_datum * to store result
+ * @return 1 on success, 0 on failure
+ *
+ * dat->data must be o_free'd after use
+ * The nul terminator is not included in dat->data.
+ */
+int o_base64_decode_alloc(const unsigned char * src, size_t len, struct _o_datum * dat);
+
+/**
+ * o_base64url_encode_alloc - Base64url encode (url format) - allocate data result in a struct _o_datum * 
+ * @param src Data to be encoded
+ * @param len Length of the data to be encoded
+ * @param dat Pointer to a struct _o_datum * to store result
+ * @return 1 on success, 0 on failure
+ *
+ * dat->data must be o_free'd after use
+ * The nul terminator is not included in dat->data.
+ */
+int o_base64url_encode_alloc(const unsigned char * src, size_t len, struct _o_datum * dat);
+
+/**
+ * o_base64url_decode_alloc - Base64 decode (url format) - allocate data result in a struct _o_datum * 
+ * @param src Data to be decoded
+ * @param len Length of the data to be decoded
+ * @param dat Pointer to a struct _o_datum * to store result
+ * @return 1 on success, 0 on failure
+ *
+ * dat->data must be o_free'd after use
+ * The nul terminator is not included in dat->data.
+ */
+int o_base64url_decode_alloc(const unsigned char * src, size_t len, struct _o_datum * dat);
+
+/**
+ * o_base64url_2_base64_alloc - Convert a base64 url format to base64 format - allocate data result in a struct _o_datum * 
+ * @param src Data to be decoded
+ * @param len Length of the data to be decoded
+ * @param dat Pointer to a struct _o_datum * to store result
+ * @return 1 on success, 0 on failure
+ *
+ * dat->data must be o_free'd after use
+ * The nul terminator is not included in dat->data.
+ */
+int o_base64url_2_base64_alloc(const unsigned char * src, size_t len, struct _o_datum * dat);
+
+/**
+ * o_base64_2_base64url_alloc - Convert a base64 format to base64 url format - allocate data result in a struct _o_datum * 
+ * @param src Data to be decoded
+ * @param len Length of the data to be decoded
+ * @param dat Pointer to a struct _o_datum * to store result
+ * @return 1 on success, 0 on failure
+ *
+ * dat->data must be o_free'd after use
+ * The nul terminator is not included in dat->data.
+ */
+int o_base64_2_base64url_alloc(const unsigned char * src, size_t len, struct _o_datum * dat);
+
 /**
  * @}
  */
