@@ -38,6 +38,11 @@
 #define strcasecmp _stricmp
 #endif
 
+#ifdef _MSC_VER
+#include <BaseTsd.h>
+typedef SSIZE_T ssize_t;
+#endif
+
 /**
  * 
  * Orcania library
@@ -47,13 +52,6 @@
  * 
  */
 
-/**
- * char * str_replace(const char * source, char * str_old, char * str_new)
- * replace all occurences of str_old by str_new in the string source
- * return a char * with the str_new value
- * return NULL on error
- * returned value must be free'd after use
- */
 char * str_replace(const char * source, const char * str_old, const char * str_new) {
   char * to_return, * ptr, * pre, * next;
   size_t len, pre_len;
@@ -66,8 +64,8 @@ char * str_replace(const char * source, const char * str_old, const char * str_n
   if (ptr == NULL) {
     return o_strdup(source);
   } else {
-    pre_len = ptr-source;
-    pre = o_malloc((pre_len+1)*sizeof(char));
+    pre_len = (size_t)(ptr-source);
+    pre = o_malloc((pre_len+1));
     if (pre == NULL) {
       return NULL;
     }
@@ -79,8 +77,8 @@ char * str_replace(const char * source, const char * str_old, const char * str_n
       o_free(pre);
       return NULL;
     }
-    len = ((ptr-source)+o_strlen(str_new)+o_strlen(next));
-    to_return = o_malloc((len+1)*sizeof(char));
+    len = (((size_t)(ptr-source))+o_strlen(str_new)+o_strlen(next));
+    to_return = o_malloc((len+1));
     if (to_return == NULL) {
       o_free(pre);
       o_free(next);
@@ -98,12 +96,6 @@ char * str_replace(const char * source, const char * str_old, const char * str_n
   }
 }
 
-/**
- * char * msprintf(const char * message, ...)
- * Implementation of sprintf that return a malloc'd char * with the string construction
- * because life is too short to use 3 lines instead of 1
- * but don't forget to free the returned value after use!
- */
 char * msprintf(const char * message, ...) {
   va_list argp, argp_cpy;
   size_t out_len = 0;
@@ -111,55 +103,43 @@ char * msprintf(const char * message, ...) {
   if (message != NULL) {
     va_start(argp, message);
     va_copy(argp_cpy, argp); // We make a copy because in some architectures, vsnprintf can modify argp
-    out_len = vsnprintf(NULL, 0, message, argp);
-    out = o_malloc(out_len+sizeof(char));
+    out_len = (size_t)vsnprintf(NULL, 0, message, argp);
+    out = o_malloc(out_len+1);
     if (out == NULL) {
       va_end(argp);
       va_end(argp_cpy);
       return NULL;
     }
-    vsnprintf(out, (out_len+sizeof(char)), message, argp_cpy);
+    vsnprintf(out, (out_len+1), message, argp_cpy);
     va_end(argp);
     va_end(argp_cpy);
   }
   return out;
 }
 
-/**
- * char * mstrcatf((char * source, const char * message, ...)
- * A combination of strcat and msprintf that will concat source and message formatted
- * and return the combination as a new allocated char *
- * and will o_free source
- * but don't forget to free the returned value after use!
- */
 char * mstrcatf(char * source, const char * message, ...) {
   va_list argp, argp_cpy;
   char * out = NULL, * message_formatted = NULL;
-  size_t message_formatted_len = 0, out_len = 0, source_len = 0;
+  size_t message_formatted_len = 0, out_len = 0;
   
   if (message != NULL) {
     va_start(argp, message);
     va_copy(argp_cpy, argp); // We make a copy because in some architectures, vsnprintf can modify argp
     if (source != NULL) {
-      source_len = o_strlen(source);
-      message_formatted_len = vsnprintf(NULL, 0, message, argp);
-      message_formatted = o_malloc(message_formatted_len+sizeof(char));
+      message_formatted_len = (size_t)vsnprintf(NULL, 0, message, argp);
+      message_formatted = o_malloc(message_formatted_len+1);
       if (message_formatted != NULL) {
-        out = o_malloc(source_len+message_formatted_len+sizeof(char));
-        vsnprintf(message_formatted, (message_formatted_len+sizeof(char)), message, argp_cpy);
-        if (out != NULL) {
-          o_strncpy(out, source, source_len);
-          o_strncpy(out+source_len, message_formatted, message_formatted_len);
-          out[source_len+message_formatted_len] = '\0';
-        }
+        memset(message_formatted, 0, message_formatted_len+1);
+        vsnprintf(message_formatted, (message_formatted_len+1), message, argp_cpy);
+        out = msprintf("%s%s", source, message_formatted);
         o_free(message_formatted);
         o_free(source);
       }
     } else {
-      out_len = vsnprintf(NULL, 0, message, argp);
-      out = o_malloc(out_len+sizeof(char));
+      out_len = (size_t)vsnprintf(NULL, 0, message, argp);
+      out = o_malloc(out_len+1);
       if (out != NULL) {
-        vsnprintf(out, (out_len+sizeof(char)), message, argp_cpy);
+        vsnprintf(out, (out_len+1), message, argp_cpy);
       }
     }
     va_end(argp);
@@ -168,20 +148,10 @@ char * mstrcatf(char * source, const char * message, ...) {
   return out;
 }
 
-/**
- * o_strdup
- * a modified strdup function that don't crash when source is NULL, instead return NULL
- * Returned value must be free'd after use
- */
 char * o_strdup(const char * source) {
   return (source==NULL?NULL:o_strndup(source, strlen(source)));
 }
 
-/**
- * o_strndup
- * a modified strndup function that don't crash when source is NULL, instead return NULL
- * Returned value must be free'd after use
- */
 char * o_strndup(const char * source, size_t len) {
   char *new_str;
 
@@ -199,10 +169,6 @@ char * o_strndup(const char * source, size_t len) {
   }
 }
 
-/**
- * o_strcmp
- * a modified strcmp function that don't crash when p1 is NULL or p2 us NULL
- */
 int o_strcmp(const char * p1, const char * p2) {
   if (p1 == NULL && p2 == NULL) {
     return 0;
@@ -215,10 +181,6 @@ int o_strcmp(const char * p1, const char * p2) {
   }
 }
 
-/**
- * o_strncmp
- * a modified strncmp function that don't crash when p1 is NULL or p2 us NULL
- */
 int o_strncmp(const char * p1, const char * p2, size_t n) {
   if ((p1 == NULL && p2 == NULL) || n <= 0) {
     return 0;
@@ -231,10 +193,6 @@ int o_strncmp(const char * p1, const char * p2, size_t n) {
   }
 }
 
-/**
- * o_strcpy
- * a modified strcpy function that don't crash when p1 is NULL or p2 us NULL
- */
 char * o_strcpy(char * p1, const char * p2) {
   if (p1 == NULL || p2 == NULL) {
     return NULL;
@@ -243,10 +201,6 @@ char * o_strcpy(char * p1, const char * p2) {
   }
 }
 
-/**
- * o_strncpy
- * a modified strncpy function that don't crash when p1 is NULL or p2 us NULL
- */
 char * o_strncpy(char * p1, const char * p2, size_t n) {
   if (p1 == NULL || p2 == NULL) {
     return NULL;
@@ -255,10 +209,6 @@ char * o_strncpy(char * p1, const char * p2, size_t n) {
   }
 }
 
-/**
- * o_strcasecmp
- * a modified strcasecmp function that don't crash when p1 is NULL or p2 us NULL
- */
 int o_strcasecmp(const char * p1, const char * p2) {
   if (p1 == NULL && p2 == NULL) {
     return 0;
@@ -271,10 +221,6 @@ int o_strcasecmp(const char * p1, const char * p2) {
   }
 }
 
-/**
- * o_strncasecmp
- * a modified strncasecmp function that don't crash when p1 is NULL or p2 us NULL
- */
 int o_strncasecmp(const char * p1, const char * p2, size_t n) {
   if ((p1 == NULL && p2 == NULL) || n <= 0) {
     return 0;
@@ -287,10 +233,6 @@ int o_strncasecmp(const char * p1, const char * p2, size_t n) {
   }
 }
 
-/**
- * o_strstr
- * a modified strstr function that don't crash when haystack is NULL or needle us NULL
- */
 char * o_strstr(const char * haystack, const char * needle) {
   if (haystack == NULL || needle == NULL) {
     return NULL;
@@ -299,10 +241,6 @@ char * o_strstr(const char * haystack, const char * needle) {
   }
 }
 
-/**
- * o_strchr
- * a modified strchr function that don't crash when haystack is NULL
- */
 char * o_strchr(const char * haystack, int c) {
   if (haystack == NULL) {
     return NULL;
@@ -311,10 +249,6 @@ char * o_strchr(const char * haystack, int c) {
   }
 }
 
-/**
- * o_strnchr
- * a modified strnchr function that don't crash when haystack is NULL
- */
 const char * o_strnchr(const char * haystack, size_t len, char c) {
   size_t offset;
   if (haystack != NULL && len > 0) {
@@ -339,14 +273,10 @@ char * o_strrchr(const char * haystack, int c) {
   }
 }
 
-/**
- * o_strrnchr
- * a modified strrnchr function that don't crash when haystack is NULL
- */
 const char * o_strrnchr(const char * haystack, size_t len, char c) {
   ssize_t offset;
   if (haystack != NULL && len > 0) {
-    for (offset = o_strlen(haystack)-1; offset>=0 && (o_strlen(haystack) - 1 - offset < len); offset--) {
+    for (offset = ((ssize_t)o_strlen(haystack))-1; offset>=0 && (o_strlen(haystack) - 1 - (size_t)offset < len); offset--) {
       if (haystack[offset] == c) {
         return (haystack+offset);
       }
@@ -383,17 +313,13 @@ static char *strcasestr(const char *haystack, const char *needle) {
   n = o_strlen(needle);
   while (*haystack) {
     if (!strnicmp(haystack++, needle, n)) {
-      return (char *)(haystack-sizeof(char));
+      return (char *)(haystack-1);
     }
   }
   return NULL;
 }
 #endif
 
-/**
- * o_strnstr
- * a modified strnstr function that don't crash when haystack is NULL or needle us NULL
- */
 char * o_strnstr(const char * haystack, const char * needle, size_t len) {
   if (haystack == NULL || needle == NULL) {
     return NULL;
@@ -402,10 +328,6 @@ char * o_strnstr(const char * haystack, const char * needle, size_t len) {
   }
 }
 
-/**
- * o_strcasestr
- * a modified strcasestr function that don't crash when haystack is NULL or needle us NULL
- */
 char * o_strcasestr(const char * haystack, const char * needle) {
   if (haystack == NULL || needle == NULL) {
     return NULL;
@@ -414,10 +336,6 @@ char * o_strcasestr(const char * haystack, const char * needle) {
   }
 }
 
-/**
- * o_strlen
- * a modified version of strlen that don't crash when s is NULL
- */
 size_t o_strlen(const char * s) {
   if (s == NULL) {
     return 0;
@@ -426,22 +344,10 @@ size_t o_strlen(const char * s) {
   }
 }
 
-/**
- * o_strnullempty
- * return true if s is NULL or empty string, false otherwise
- */
 int o_strnullempty(const char * s) {
   return (s == NULL || s[0] == '\0');
 }
 
-/**
- * Split a string into an array of strings using separator string
- * return the number of elements to be returned, 0 on error
- * if return_array is not NULL, set the returned array in it
- * return_array is an array of char * ending with a NULL value
- * return_array must be free'd after use
- * you can use free_string_array to free return_array
- */
 size_t split_string(const char * string, const char * separator, char *** return_array) {
   size_t result = 0;
   char * token;
@@ -457,7 +363,7 @@ size_t split_string(const char * string, const char * separator, char *** return
       if (return_array != NULL) {
         (*return_array) = o_realloc((*return_array), (result + 1)*sizeof(char*));
         if ((*return_array) != NULL) {
-          (*return_array)[result-1] = o_strndup(begin, (token-begin));
+          (*return_array)[result-1] = o_strndup(begin, (size_t)(token-begin));
           (*return_array)[result] = NULL;
         }
       }
@@ -476,9 +382,6 @@ size_t split_string(const char * string, const char * separator, char *** return
   return result;
 }
 
-/**
- * Clean an array of strings
- */
 void free_string_array(char ** array) {
   int i;
   if (array != NULL) {
@@ -490,9 +393,6 @@ void free_string_array(char ** array) {
   }
 }
 
-/**
- * Count the number of elements in an array of strings
- */
 size_t string_array_size(char ** array) {
   size_t ret = 0;
   if (array != NULL) {
@@ -501,9 +401,6 @@ size_t string_array_size(char ** array) {
   return ret;
 }
 
-/**
- * Join a string array into a single string
- */
 char * string_array_join(const char ** array, const char * separator) {
   char * to_return = NULL, * tmp;
   int i;
@@ -522,9 +419,6 @@ char * string_array_join(const char ** array, const char * separator) {
   return to_return;
 }
 
-/**
- * Remove string of beginning and ending whitespaces
- */
 const char * trimwhitespace(char * str) {
   char * end;
 
@@ -546,9 +440,6 @@ const char * trimwhitespace(char * str) {
   return str;
 }
 
-/**
- * Remove string of beginning and ending given character
- */
 char * trimcharacter(char * str, char to_remove) {
   char * end;
 
@@ -570,9 +461,6 @@ char * trimcharacter(char * str, char to_remove) {
   return str;
 }
 
-/**
- * Check if an array of string has a specified value, case sensitive
- */
 int string_array_has_value(const char ** array, const char * needle) {
   int i;
   if (array != NULL && needle != NULL) {
@@ -587,9 +475,6 @@ int string_array_has_value(const char ** array, const char * needle) {
   }
 }
 
-/**
- * Check if an array of string has a specified value, case insensitive
- */
 int string_array_has_value_case(const char ** array, const char * needle) {
   int i;
   if (array != NULL && needle != NULL) {
@@ -604,9 +489,6 @@ int string_array_has_value_case(const char ** array, const char * needle) {
   }
 }
 
-/**
- * Check if an array of string has a specified value, case sensitive, limit to len characters
- */
 int string_array_has_value_n(const char ** array, const char * needle, size_t len) {
   int i;
   if (array != NULL && needle != NULL) {
@@ -621,9 +503,6 @@ int string_array_has_value_n(const char ** array, const char * needle, size_t le
   }
 }
 
-/**
- * Check if an array of string has a specified value, case insensitive, limit to len characters
- */
 int string_array_has_value_ncase(const char ** array, const char * needle, size_t len) {
   int i;
   if (array != NULL && needle != NULL) {
@@ -638,9 +517,6 @@ int string_array_has_value_ncase(const char ** array, const char * needle, size_
   }
 }
 
-/**
- * Check if an array of string has a specified trimmed value
- */
 int string_array_has_trimmed_value(const char ** array, const char * needle) {
   int i, to_return = 0;
   char * duplicate_needle, * duplicate_value;
@@ -667,10 +543,6 @@ int string_array_has_trimmed_value(const char ** array, const char * needle) {
   return to_return;
 }
 
-/**
- * pointer_list_init
- * Initialize a pointer list structure
- */
 void pointer_list_init(struct _pointer_list * pointer_list) {
   if (pointer_list != NULL) {
     pointer_list->size = 0;
@@ -678,10 +550,6 @@ void pointer_list_init(struct _pointer_list * pointer_list) {
   }
 }
 
-/**
- * pointer_list_clean
- * Clean a pointer list structure
- */
 void pointer_list_clean(struct _pointer_list * pointer_list) {
   size_t i;
   if (pointer_list != NULL) {
@@ -691,11 +559,6 @@ void pointer_list_clean(struct _pointer_list * pointer_list) {
   }
 }
 
-/**
- * pointer_list_clean_free
- * Clean a pointer list structure
- * Free all elements using the free_function given in parameters
- */
 void pointer_list_clean_free(struct _pointer_list * pointer_list, void (* free_function)(void * elt)) {
   size_t i;
   if (pointer_list != NULL) {
@@ -706,10 +569,6 @@ void pointer_list_clean_free(struct _pointer_list * pointer_list, void (* free_f
   }
 }
 
-/**
- * pointer_list_size
- * Return the size of a pointer list
- */
 size_t pointer_list_size(struct _pointer_list * pointer_list) {
   if (pointer_list != NULL) {
     return pointer_list->size;
@@ -718,11 +577,6 @@ size_t pointer_list_size(struct _pointer_list * pointer_list) {
   }
 }
 
-/**
- * pointer_list_append
- * Appends an element at the end of a pointer list
- * Return 1 on success, 0 on error
- */
 int pointer_list_append(struct _pointer_list * pointer_list, void * element) {
   if (pointer_list) {
     pointer_list->list = o_realloc(pointer_list->list, (pointer_list->size+1)*sizeof(void *));
@@ -740,10 +594,6 @@ int pointer_list_append(struct _pointer_list * pointer_list, void * element) {
   }
 }
 
-/**
- * pointer_list_get_at
- * Returns an element of a pointer list at the specified index or NULL if non valid index
- */
 void * pointer_list_get_at(struct _pointer_list * pointer_list, size_t index) {
   if (pointer_list != NULL && index < pointer_list->size) {
     return pointer_list->list[index];
@@ -752,11 +602,6 @@ void * pointer_list_get_at(struct _pointer_list * pointer_list, size_t index) {
   }
 }
 
-/**
- * pointer_list_remove_at
- * Removes an element of a pointer list at the specified index
- * Return 1 on success, 0 on error or non valid index
- */
 int pointer_list_remove_at(struct _pointer_list * pointer_list, size_t index) {
   size_t i;
   if (pointer_list != NULL && index < pointer_list->size) {
@@ -776,12 +621,6 @@ int pointer_list_remove_at(struct _pointer_list * pointer_list, size_t index) {
   }
 }
 
-/**
- * pointer_list_remove_at_free
- * Removes an element of a pointer list at the specified index
- * Return 1 on success, 0 on error or non valid index
- * Free the element using the free_function given in parameters
- */
 int pointer_list_remove_at_free(struct _pointer_list * pointer_list, size_t index, void (* free_function)(void * elt)) {
   if (pointer_list != NULL && index < pointer_list->size) {
     free_function(pointer_list_get_at(pointer_list, index));
@@ -791,11 +630,6 @@ int pointer_list_remove_at_free(struct _pointer_list * pointer_list, size_t inde
   }
 }
 
-/**
- * pointer_list_insert_at
- * Inserts an element at the specified index of a pointer list
- * Return 1 on success, 0 on error or non valid index
- */
 int pointer_list_insert_at(struct _pointer_list * pointer_list, void * element, size_t index) {
   size_t i;
   if (pointer_list != NULL && index <= pointer_list->size) {
@@ -815,11 +649,6 @@ int pointer_list_insert_at(struct _pointer_list * pointer_list, void * element,
   }
 }
 
-/**
- * pointer_list_remove_at
- * Removes an element of a pointer list corresponding to the specified element
- * Return 1 on success, 0 on error or non valid element
- */
 int pointer_list_remove_pointer(struct _pointer_list * pointer_list, void * element) {
   size_t index;
   if (pointer_list != NULL) {
@@ -834,12 +663,6 @@ int pointer_list_remove_pointer(struct _pointer_list * pointer_list, void * elem
   }
 }
 
-/**
- * pointer_list_remove_pointer_free
- * Removes an element of a pointer list corresponding to the specified element
- * Free the element using the free_function given in parameters
- * Return 1 on success, 0 on error or non valid element
- */
 int pointer_list_remove_pointer_free(struct _pointer_list * pointer_list, void * element, void (* free_function)(void * elt)) {
   size_t index;
   if (pointer_list != NULL) {
